@@ -8,10 +8,10 @@ namespace Qyoto {
 
     public partial class QVariant : Object, IDisposable {
 
-        [DllImport("libqyoto", CharSet=CharSet.Ansi)]
+        [DllImport("qyoto", CharSet=CharSet.Ansi)]
         static extern IntPtr QVariantValue(string typeName, IntPtr variant);
 
-        [DllImport("libqyoto", CharSet=CharSet.Ansi)]
+        [DllImport("qyoto", CharSet=CharSet.Ansi)]
         static extern IntPtr QVariantFromValue(int type, IntPtr value);
 
         public T Value<T>() {
@@ -97,7 +97,7 @@ namespace Qyoto {
             } else if (typeof(T) == typeof(QByteArray)) {
                 return new QVariant((QByteArray) value);
             } else if (typeof(T) == typeof(char)) {
-                return new QVariant((char) value);
+                return new QVariant(new QChar((char) value));
             } else if (typeof(T) == typeof(QDate)) {
                 return new QVariant((QDate) value);
             } else if (typeof(T) == typeof(QDateTime)) {
@@ -228,6 +228,9 @@ namespace Qyoto {
         public static implicit operator QVariant(QPolygon arg) {
             return QVariant.FromValue<QPolygon>(arg);
         }
+        public static implicit operator QVariant(QRect arg) {
+            return QVariant.FromValue<QRect>(arg);
+        }
         public static implicit operator QVariant(QRegion arg) {
             return QVariant.FromValue<QRegion>(arg);
         }
@@ -309,6 +312,9 @@ namespace Qyoto {
         public static implicit operator QPolygon(QVariant arg) {
             return arg.Value<QPolygon>();
         }
+        public static implicit operator QRect(QVariant arg) {
+            return arg.Value<QRect>();
+        }
         public static implicit operator QRegion(QVariant arg) {
             return arg.Value<QRegion>();
         }
