@@ -64,15 +64,19 @@
 #define QYOTO_VERSION "0.0.1"
 // #define DEBUG
 
-#include <qt_smoke.h>
-
-// Maps from a classname in the form Qt::Widget to an int id
-QHash<int,char *> classname;
+#include <qtcore_smoke.h>
+#include <qtgui_smoke.h>
+#include <qtxml_smoke.h>
+#include <qtsql_smoke.h>
+#include <qtopengl_smoke.h>
+#include <qtnetwork_smoke.h>
+#include <qtsvg_smoke.h>
+#include <qtdbus_smoke.h>
 
 extern bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
 extern bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
 
-extern TypeHandler Qyoto_handlers[];
+extern Q_DECL_IMPORT TypeHandler Qyoto_handlers[];
 extern void install_handlers(TypeHandler *);
 
 extern bool IsContainedInstanceQt(smokeqyoto_object *o);
@@ -126,18 +130,22 @@ DebugChannel()
 Q_DECL_EXPORT Smoke::ModuleIndex 
 FindMethodId(char * classname, char * mungedname, char * signature) 
 {
-	Smoke::ModuleIndex negativeIndex = { 0, -1 };
+	static Smoke::ModuleIndex negativeIndex(0, -1);
 #ifdef DEBUG
 	printf("FindMethodId(classname: %s mungedname: %s signature: %s)\n", classname, mungedname, signature);
 	fflush(stdout);
 #endif
 
-	Smoke::ModuleIndex meth = qt_Smoke->findMethod(classname, mungedname);
+	Smoke::ModuleIndex meth = qtcore_Smoke->findMethod(classname, mungedname);
 #ifdef DEBUG
 	if (do_debug & qtdb_calls) printf("DAMNIT on %s::%s => %d\n", classname, mungedname, meth.index);
 #endif
 	if (meth.index == 0) {
-    	meth = qt_Smoke->findMethod("QGlobalSpace", mungedname);
+		foreach (Smoke *smoke, qyoto_modules.keys()) {
+			meth = smoke->findMethod("QGlobalSpace", mungedname);
+			if (meth.index)
+				break;
+		}
 #ifdef DEBUG
 		if (do_debug & qtdb_calls) printf("DAMNIT on QGlobalSpace::%s => %d\n", mungedname, meth.index);
 #endif
@@ -153,7 +161,7 @@ FindMethodId(char * classname, char * mungedname, char * signature)
 		} else if (i > 0) {	// single match
 	    	Smoke::Method &methodRef = meth.smoke->methods[i];
 			if ((methodRef.flags & Smoke::mf_internal) == 0) {
-				Smoke::ModuleIndex ret = { meth.smoke, i };
+				Smoke::ModuleIndex ret(meth.smoke, i);
 				return ret;
 			}
 		} else {		// multiple match
@@ -189,7 +197,7 @@ static QByteArray * currentSignature = 0;
 #endif
 		
 					if (*currentSignature == signature) {
-						Smoke::ModuleIndex ret = { meth.smoke,  meth.smoke->ambiguousMethodList[ambiguousId] };
+						Smoke::ModuleIndex ret(meth.smoke,  meth.smoke->ambiguousMethodList[ambiguousId]);
 						return ret;
 					}
 				}
@@ -201,6 +209,21 @@ static QByteArray * currentSignature = 0;
 	return negativeIndex;
 }
 
+static QMetaObject* get_meta_object(const char* classname) {
+    Smoke::ModuleIndex classId = Smoke::findClass(classname);
+    Smoke::ModuleIndex nameId = classId.smoke->idMethodName("staticMetaObject");
+    Smoke::ModuleIndex meth = classId.smoke->findMethod(classId, nameId);
+    if (meth.index <= 0) {
+        // Should never happen..
+    }
+
+    Smoke::Method &methodId = meth.smoke->methods[meth.smoke->methodMaps[meth.index].method];
+    Smoke::ClassFn fn = meth.smoke->classes[methodId.classId].classFn;
+    Smoke::StackItem i[1];
+    (*fn)(methodId.method, 0, i);
+    return (QMetaObject*) i[0].s_voidp;
+}
+
 /* Adapted from the internal function qt_qFindChildren() in qobject.cpp */
 Q_DECL_EXPORT void
 cs_qFindChildren_helper(const QObject *parent, const QString &name, const QRegExp *re,
@@ -223,7 +246,7 @@ cs_qFindChildren_helper(const QObject *parent, const QString &name, const QRegEx
 					list->append((*GetInstance)(obj, true));
 			}
 		}
-		qt_qFindChildren_helper(obj, name, re, mo, list);
+		cs_qFindChildren_helper(obj, name, re, mo, list);
 	}
 }
 
@@ -231,18 +254,26 @@ cs_qFindChildren_helper(const QObject *parent, const QString &name, const QRegEx
      obj.findChildren(Qt::Widget, "Optional Widget Name")
 */
 Q_DECL_EXPORT void
-FindQObjectChildren(void* parent, void* regexp, char* childName, FromIntPtr addFn)
+FindQObjectChildren(void* parent, const char *childClassName, void* childMetaObject, void* regexp, char* childName, FromIntPtr addFn)
 {
-	QMetaObject *mo = parent_meta_object(parent);
-	
 	smokeqyoto_object *o;
 	o = (smokeqyoto_object*) (*GetSmokeObject)(parent);
 	QObject* p = (QObject*) o->ptr;
-	
+
+	QMetaObject *mo = 0;
+	if (childMetaObject != 0) {
+		o = (smokeqyoto_object*) (*GetSmokeObject)(childMetaObject);
+		mo = (QMetaObject*) o->ptr;
+		(*FreeGCHandle)(childMetaObject);
+	} else {
+		mo = get_meta_object(childClassName);
+	}
+
 	QRegExp* re = 0;
 	if (regexp) {
 		o = (smokeqyoto_object*) (*GetSmokeObject)(regexp);
 		re = (QRegExp*) o->ptr;
+		(*FreeGCHandle)(regexp);
 	}
 	
 	QList<void*> *list = new QList<void*>();
@@ -257,15 +288,9 @@ FindQObjectChildren(void* parent, void* regexp, char* childName, FromIntPtr addF
 
 /* Adapted from the internal function qt_qFindChild() in qobject.cpp */
 Q_DECL_EXPORT void *
-cs_qFindChildHelper(void * parent, const QString &name, const QMetaObject &mo)
+cs_qFindChild_helper(QObject* parent, const QString &name, const QMetaObject &mo)
 {
-	if (!parent)
-		return 0;
-		
-	smokeqyoto_object *o = (smokeqyoto_object*) (*GetSmokeObject)(parent);
-	QObject* p = (QObject*) o->ptr;
-	
-	const QObjectList &children = p->children();
+	const QObjectList &children = parent->children();
 	QObject* obj;
 	void* monoObject;
 	
@@ -278,7 +303,7 @@ cs_qFindChildHelper(void * parent, const QString &name, const QMetaObject &mo)
 		}
 	}
 	for (i = 0; i < children.size(); ++i) {
-		monoObject = qt_qFindChild_helper(children.at(i), name, mo);
+		monoObject = cs_qFindChild_helper(children.at(i), name, mo);
 		if (monoObject)
 			return monoObject;
 	}
@@ -286,13 +311,22 @@ cs_qFindChildHelper(void * parent, const QString &name, const QMetaObject &mo)
 }
 
 Q_DECL_EXPORT void *
-FindQObjectChild(void* parent, char * childName)
+FindQObjectChild(void* parent, const char* childClassName, void* childMetaObject, char * childName)
 {
-	QString name = QString::fromUtf8(childName);
-	
-	QMetaObject * mo = parent_meta_object(parent);
+	smokeqyoto_object *o = (smokeqyoto_object*) (*GetSmokeObject)(parent);
+	QObject *p = (QObject*) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject", true).index);
 	(*FreeGCHandle)(parent);
-	return cs_qFindChildHelper(parent, name, *mo);
+
+	QMetaObject *mo = 0;
+	if (childMetaObject != 0) {
+		o = (smokeqyoto_object*) (*GetSmokeObject)(childMetaObject);
+		mo = (QMetaObject*) o->ptr;
+		(*FreeGCHandle)(childMetaObject);
+	} else {
+		mo = get_meta_object(childClassName);
+	}
+
+	return cs_qFindChild_helper(p, childName, *mo);
 }
 
 Q_DECL_EXPORT void*
@@ -310,8 +344,8 @@ qyoto_qt_metacast(void* obj, char* target)
 #endif
 		return instance;
 	}
-	Smoke* s = Smoke::classMap[target];
-	smokeqyoto_object* to = alloc_smokeqyoto_object(false, s, s->idClass(target).index, ret);
+	Smoke::ModuleIndex mi = Smoke::classMap[target];
+	smokeqyoto_object* to = alloc_smokeqyoto_object(false, mi.smoke, mi.index, ret);
 	instance = (*CreateInstance)(qyoto_resolve_classname(to), to);
 	mapPointer(instance, to, to->classId, 0);
 #ifdef DEBUG
@@ -377,7 +411,7 @@ QVariantFromValue(int type, void * value)
 	smokeqyoto_object *o = (smokeqyoto_object*) (*GetSmokeObject)(value);
 	if (o) (*FreeGCHandle)(value);
 	QVariant * v = new QVariant(type, o? o->ptr : value);
-	Smoke::ModuleIndex id = qt_Smoke->findClass("QVariant");
+	Smoke::ModuleIndex id = qtcore_Smoke->findClass("QVariant");
 	smokeqyoto_object  * vo = alloc_smokeqyoto_object(true, id.smoke, id.index, (void *) v);
 	return (*CreateInstance)("Qyoto.QVariant", vo);
 }
@@ -884,7 +918,7 @@ SignalEmit(char * signature, char * type, void * obj, Smoke::StackItem * sp, int
 }
 
 Q_DECL_EXPORT void*
-qyoto_make_metaObject(void* obj, void* parentMeta, const char* stringdata, int stringdata_count,
+qyoto_make_metaObject(const char* parentClassName, void* parentMeta, const char* stringdata, int stringdata_count,
                 const uint* data, int data_count)
 {
 	QMetaObject* parent = 0;
@@ -892,7 +926,7 @@ qyoto_make_metaObject(void* obj, void* parentMeta, const char* stringdata, int s
 	if (parentMeta == 0) {
 		// The parent class is a Smoke class, so call metaObject() on the
 		// instance to get it via a smoke library call
-		parent = parent_meta_object(obj);
+		parent = get_meta_object(parentClassName);
 	} else {
 		// The parent class is a custom C# class whose metaObject
 		// was constructed at runtime
@@ -901,8 +935,6 @@ qyoto_make_metaObject(void* obj, void* parentMeta, const char* stringdata, int s
 		(*FreeGCHandle)(parentMeta);
 	}
 
-	(*FreeGCHandle)(obj);
-
 	char* my_stringdata = new char[stringdata_count];
 	memcpy(my_stringdata, stringdata, stringdata_count * sizeof(char));
 	
@@ -996,36 +1028,52 @@ qyoto_make_metaObject(void* obj, void* parentMeta, const char* stringdata, int s
 	
 	// create smoke object
 	smokeqyoto_object  * m = alloc_smokeqyoto_object(	true, 
-														qt_Smoke, 
-														qt_Smoke->idClass("QMetaObject").index, 
+														qtcore_Smoke,
+														qtcore_Smoke->idClass("QMetaObject").index,
 														meta );
 	
 	// create wrapper C# instance
 	return (*CreateInstance)("Qyoto.QMetaObject", m);
 }
 
-static Qyoto::Binding binding;
+#define INIT_BINDING(module) \
+    static QHash<int,char *> module##_classname; \
+    for (int i = 1; i <= module##_Smoke->numClasses; i++) { \
+        QByteArray name(module##_Smoke->classes[i].className); \
+        name.replace("::", "."); \
+        if (name != "QAccessible2" && name != "QDBus" && name != "QGL" && name != "QSql" && name != "QSsl") { \
+            name = prefix + name; \
+        } \
+        module##_classname.insert(i, strdup(name.constData())); \
+    } \
+    static Qyoto::Binding module##_binding = Qyoto::Binding(module##_Smoke, &module##_classname); \
+    QyotoModule module = { "Qyoto_" #module, qyoto_resolve_classname_qt, IsContainedInstanceQt, &module##_binding }; \
+    qyoto_modules[module##_Smoke] = module;
+
 
 Q_DECL_EXPORT void
 Init_qyoto()
 {
-    init_qt_Smoke();
+    init_qtcore_Smoke();
+    init_qtgui_Smoke();
+    init_qtxml_Smoke();
+    init_qtsql_Smoke();
+    init_qtopengl_Smoke();
+    init_qtnetwork_Smoke();
+    init_qtsvg_Smoke();
+    init_qtdbus_Smoke();
+
     qyoto_install_handlers(Qyoto_handlers);
     QByteArray prefix("Qyoto.");
 
-    for (int i = 1; i <= qt_Smoke->numClasses; i++) {
-		QByteArray name(qt_Smoke->classes[i].className);
-		name.replace("::", ".");
-		if (name != "QAccessible2" && name != "QDBus" && name != "QGL" && name != "QSql" && name != "QSsl") {
-			name = prefix + name;
-		}
-
-		classname.insert(i, strdup(name.constData()));
-    }
-
-    binding = Qyoto::Binding(qt_Smoke, &classname);
-	QyotoModule module = { "Qyoto", qyoto_resolve_classname_qt, IsContainedInstanceQt, &binding };
-    qyoto_modules[qt_Smoke] = module;
+    INIT_BINDING(qtcore)
+    INIT_BINDING(qtgui)
+    INIT_BINDING(qtxml)
+    INIT_BINDING(qtsql)
+    INIT_BINDING(qtopengl)
+    INIT_BINDING(qtnetwork)
+    INIT_BINDING(qtsvg)
+    INIT_BINDING(qtdbus)
 
 #if QT_VERSION >= 0x40300
     QInternal::registerCallback(QInternal::EventNotifyCallback, qyoto_event_notify);
