@@ -2,7 +2,7 @@
 #include <QtGui>
 
 #include <smoke/smoke.h>
-#include <smoke/qt_smoke.h>
+#include <smoke/qtcore_smoke.h>
 
 #include "smokeqyoto.h"
 #include "qyoto.h"
@@ -304,14 +304,15 @@ void unmapPointer(smokeqyoto_object *o, Smoke::Index classId, void *lastptr) {
 const char *
 qyoto_resolve_classname(smokeqyoto_object * o)
 {
-	if (o->smoke->isDerivedFromByName(o->smoke->classes[o->classId].className, "QObject")) {
+	if (o->smoke->isDerivedFrom(o->smoke->classes[o->classId].className, "QObject")) {
 		QObject * qobject = (QObject *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject").index);
 		const QMetaObject * meta = qobject->metaObject();
 
 		while (meta != 0) {
-			o->smoke = Smoke::classMap[meta->className()];
+			Smoke::ModuleIndex mi = Smoke::classMap[meta->className()];
+			o->smoke = mi.smoke;
+			o->classId = mi.index;
 			if (o->smoke != 0) {
-				o->classId = o->smoke->idClass(meta->className()).index;
 				if (o->classId != 0) {
 					if (strcmp(o->smoke->classes[o->classId].className, "QAbstractItemModel") == 0)
 						return "Qyoto.QItemModel";
@@ -408,7 +409,7 @@ DisconnectDelegate(void* obj, const char* signal, void* delegate)
 Q_DECL_EXPORT QMetaObject* parent_meta_object(void* obj) {
 	smokeqyoto_object* o = (smokeqyoto_object*) (*GetSmokeObject)(obj);
 	Smoke::ModuleIndex nameId = o->smoke->findMethodName(o->smoke->className(o->classId), "metaObject");
-	Smoke::ModuleIndex classId = { o->smoke, o->classId };
+	Smoke::ModuleIndex classId(o->smoke, o->classId);
 	Smoke::ModuleIndex meth = o->smoke->findMethod(classId, nameId);
 	if (meth.index <= 0) {
 		// Should never happen..
@@ -421,6 +422,8 @@ Q_DECL_EXPORT QMetaObject* parent_meta_object(void* obj) {
 	return (QMetaObject*) i[0].s_voidp;
 }
 
+}
+
 QList<MocArgument*>
 GetMocArguments(Smoke* smoke, const char * typeName, QList<QByteArray> methodTypes)
 {
@@ -480,36 +483,36 @@ static QRegExp * rx = 0;
 				}
 			} else if (staticType == "bool") {
 				arg->argType = xmoc_bool;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "int") {
 				arg->argType = xmoc_int;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "uint") {
 				arg->argType = xmoc_uint;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "long") {
 				arg->argType = xmoc_long;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "ulong") {
 				arg->argType = xmoc_ulong;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "double") {
 				arg->argType = xmoc_double;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "char*") {
 				arg->argType = xmoc_charstar;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "QString") {
 				arg->argType = xmoc_QString;
 				name += "*";
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			}
 
@@ -530,7 +533,7 @@ int qt_metacall(void* obj, int _c, int _id, void* _o) {
 	smokeqyoto_object* o = (smokeqyoto_object*) (*GetSmokeObject)(obj);
 	
 	Smoke::ModuleIndex nameId = o->smoke->findMethodName(o->smoke->className(o->classId), "qt_metacall$$?");
-	Smoke::ModuleIndex classId = { o->smoke, o->classId };
+	Smoke::ModuleIndex classId(o->smoke, o->classId);
 	Smoke::ModuleIndex meth = nameId.smoke->findMethod(classId, nameId);
 	if(meth.index > 0) {
 		Smoke::Method &m = meth.smoke->methods[meth.smoke->methodMaps[meth.index].method];
@@ -585,13 +588,11 @@ int qt_metacall(void* obj, int _c, int _id, void* _o) {
 		((void**)_o)[0] = sqo->ptr;
 	} else if (_c == QMetaObject::WriteProperty) {
 		QMetaProperty property = metaobject->property(_id);
-		smokeqyoto_object* sqo = alloc_smokeqyoto_object(false, qt_Smoke,
-		                                                 qt_Smoke->idClass("QVariant").index,
+		smokeqyoto_object* sqo = alloc_smokeqyoto_object(false, qtcore_Smoke,
+		                                                 qtcore_Smoke->idClass("QVariant").index,
 		                                                 ((void**)_o)[0]);
 		void* variant = (*CreateInstance)("Qyoto.QVariant", sqo);
 		(*SetProperty)(obj, property.name(), variant);
 	}
 	return _id - count;
 }
-
-}
