@@ -63,7 +63,7 @@
 #include <ruby.h>
 
 #include <smoke/smoke.h>
-#include <smoke/qt_smoke.h>
+#include <smoke/qtcore_smoke.h>
 
 #undef free
 #undef malloc
@@ -112,7 +112,7 @@ QHash<Smoke::ModuleIndex, QByteArray*> IdToClassNameMap;
 
 #define logger logger_backend
 
-Smoke::ModuleIndex _current_method = { 0, 0 };
+Smoke::ModuleIndex _current_method;
 
 smokeruby_object * 
 alloc_smokeruby_object(bool allocated, Smoke * smoke, int classId, void * ptr)
@@ -241,7 +241,7 @@ Binding::callMethod(Smoke::Index method, void *ptr, Smoke::Stack args, bool /*is
 	smokeruby_object *o = value_obj_info(obj);
 
 	if (do_debug & qtdb_virtual) {
-		Smoke::Method & meth = smoke->methods[method];
+		const Smoke::Method & meth = smoke->methods[method];
 		QByteArray signature(smoke->methodNames[meth.name]);
 		signature += "(";
 			for (int i = 0; i < meth.numArgs; i++) {
@@ -269,7 +269,8 @@ Binding::callMethod(Smoke::Index method, void *ptr, Smoke::Stack args, bool /*is
 		methodName += (sizeof("operator") - 1);
 	}
 		// If the virtual method hasn't been overriden, just call the C++ one.
-	if (rb_respond_to(obj, rb_intern(methodName)) == 0) {
+		// During GC, avoid checking for override and just call the C++ version
+	if (rb_during_gc() || rb_respond_to(obj, rb_intern(methodName)) == 0) {
     	return false;
 	}
 	QtRuby::VirtualMethodCall c(smoke, method, args, obj, ALLOCA_N(VALUE, smoke->methods[method].numArgs));
@@ -279,7 +280,7 @@ Binding::callMethod(Smoke::Index method, void *ptr, Smoke::Stack args, bool /*is
 
 char*
 Binding::className(Smoke::Index classId) {
-	Smoke::ModuleIndex mi = { smoke, classId };
+	Smoke::ModuleIndex mi(smoke, classId);
 	return (char *) (const char *) *(IdToClassNameMap.value(mi));
 }
 
@@ -460,14 +461,15 @@ static char p[CAT_BUFFER_SIZE];
 const char *
 resolve_classname(smokeruby_object * o)
 {
-	if (o->smoke->isDerivedFromByName(o->smoke->classes[o->classId].className, "QObject")) {
+	if (Smoke::isDerivedFrom(o->smoke->classes[o->classId].className, "QObject")) {
 		QObject * qobject = (QObject *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject").index);
 		const QMetaObject * meta = qobject->metaObject();
 
 		while (meta != 0) {
-			o->smoke = Smoke::classMap[meta->className()];
+			Smoke::ModuleIndex mi = o->smoke->findClass(meta->className());
+			o->smoke = mi.smoke;
+			o->classId = mi.index;
 			if (o->smoke != 0) {
-				o->classId = o->smoke->idClass(meta->className()).index;
 				if (o->classId != 0) {
 					return qtruby_modules[o->smoke].binding->className(o->classId);
 				}
@@ -492,9 +494,11 @@ findMethod(VALUE /*self*/, VALUE c_value, VALUE name_value)
     char *c = StringValuePtr(c_value);
     char *name = StringValuePtr(name_value);
     VALUE result = rb_ary_new();
-    Smoke* s = Smoke::classMap[c];
-    Smoke::ModuleIndex meth = qt_Smoke->NullModuleIndex;
-    if (s) meth = s->findMethod(c, name);
+    Smoke::ModuleIndex classId = Smoke::findClass(c);    
+    Smoke::ModuleIndex meth = Smoke::NullModuleIndex;
+    if (classId.smoke != 0) {
+        meth = classId.smoke->findMethod(c, name);
+    }
 #ifdef DEBUG
     if (do_debug & qtdb_calls) qWarning("Found method %s::%s => %d", c, name, meth.index);
 #endif
@@ -521,14 +525,14 @@ findMethod(VALUE /*self*/, VALUE c_value, VALUE name_value)
         if (i == 0) {		// shouldn't happen
             rb_raise(rb_eArgError, "Corrupt method %s::%s", c, name);
         } else if(i > 0) {	// single match
-            Smoke::Method &methodRef = meth.smoke->methods[i];
+            const Smoke::Method &methodRef = meth.smoke->methods[i];
             if ((methodRef.flags & Smoke::mf_internal) == 0) {
                 rb_ary_push(result, rb_funcall(moduleindex_class, rb_intern("new"), 2, INT2NUM(smokeList.indexOf(meth.smoke)), INT2NUM(i)));
             }
         } else {		// multiple match
             i = -i;		// turn into ambiguousMethodList index
             while (meth.smoke->ambiguousMethodList[i]) {
-                Smoke::Method &methodRef = meth.smoke->methods[meth.smoke->ambiguousMethodList[i]];
+                const Smoke::Method &methodRef = meth.smoke->methods[meth.smoke->ambiguousMethodList[i]];
                 if ((methodRef.flags & Smoke::mf_internal) == 0) {
                     rb_ary_push(result, rb_funcall(moduleindex_class, rb_intern("new"), 2, INT2NUM(smokeList.indexOf(meth.smoke)), INT2NUM(meth.smoke->ambiguousMethodList[i])));
 //#ifdef DEBUG
@@ -592,14 +596,14 @@ findAllMethods(int argc, VALUE * argv, VALUE /*self*/)
                     Smoke::Index ix = smoke->methodMaps[i].method;
                     VALUE meths = rb_ary_new();
                     if (ix >= 0) {	// single match
-                        Smoke::Method &methodRef = smoke->methods[ix];
+                        const Smoke::Method &methodRef = smoke->methods[ix];
                         if ((methodRef.flags & Smoke::mf_internal) == 0) {
                             rb_ary_push(meths, rb_funcall(moduleindex_class, rb_intern("new"), 2, INT2NUM(smokeList.indexOf(smoke)), INT2NUM((int) ix)));
                         }
                     } else {		// multiple match
                         ix = -ix;		// turn into ambiguousMethodList index
                         while (smoke->ambiguousMethodList[ix]) {
-                            Smoke::Method &methodRef = smoke->methods[smoke->ambiguousMethodList[ix]];
+                            const Smoke::Method &methodRef = smoke->methods[smoke->ambiguousMethodList[ix]];
                             if ((methodRef.flags & Smoke::mf_internal) == 0) {
                                 rb_ary_push(meths, rb_funcall(moduleindex_class, rb_intern("new"), 2, INT2NUM(smokeList.indexOf(smoke)), INT2NUM((int)smoke->ambiguousMethodList[ix])));
                             }
@@ -624,11 +628,11 @@ findAllMethods(int argc, VALUE * argv, VALUE /*self*/)
 
 #define PUSH_QTRUBY_METHOD		\
 		if (	(methodRef.flags & (Smoke::mf_internal|Smoke::mf_ctor|Smoke::mf_dtor)) == 0 \
-				&& strcmp(qt_Smoke->methodNames[methodRef.name], "operator=") != 0 \
-				&& strcmp(qt_Smoke->methodNames[methodRef.name], "operator!=") != 0 \
-				&& strcmp(qt_Smoke->methodNames[methodRef.name], "operator--") != 0 \
-				&& strcmp(qt_Smoke->methodNames[methodRef.name], "operator++") != 0 \
-				&& strncmp(qt_Smoke->methodNames[methodRef.name], "operator ", strlen("operator ")) != 0 \
+				&& strcmp(s->methodNames[methodRef.name], "operator=") != 0 \
+				&& strcmp(s->methodNames[methodRef.name], "operator!=") != 0 \
+				&& strcmp(s->methodNames[methodRef.name], "operator--") != 0 \
+				&& strcmp(s->methodNames[methodRef.name], "operator++") != 0 \
+				&& strncmp(s->methodNames[methodRef.name], "operator ", strlen("operator ")) != 0 \
 				&& (	(flags == 0 && (methodRef.flags & (Smoke::mf_static|Smoke::mf_enum|Smoke::mf_protected)) == 0) \
 						|| (	flags == Smoke::mf_static \
 								&& (methodRef.flags & Smoke::mf_enum) == 0 \
@@ -637,18 +641,18 @@ findAllMethods(int argc, VALUE * argv, VALUE /*self*/)
 						|| (	flags == Smoke::mf_protected \
 								&& (methodRef.flags & Smoke::mf_static) == 0 \
 								&& (methodRef.flags & Smoke::mf_protected) == Smoke::mf_protected ) ) ) { \
-			if (strncmp(qt_Smoke->methodNames[methodRef.name], "operator", strlen("operator")) == 0) { \
-				if (op_re.indexIn(qt_Smoke->methodNames[methodRef.name]) != -1) { \
+			if (strncmp(s->methodNames[methodRef.name], "operator", strlen("operator")) == 0) { \
+				if (op_re.indexIn(s->methodNames[methodRef.name]) != -1) { \
 					rb_ary_push(result, rb_str_new2((op_re.cap(1) + op_re.cap(2)).toLatin1())); \
 				} else { \
-					rb_ary_push(result, rb_str_new2(qt_Smoke->methodNames[methodRef.name] + strlen("operator"))); \
+					rb_ary_push(result, rb_str_new2(s->methodNames[methodRef.name] + strlen("operator"))); \
 				} \
-			} else if (predicate_re.indexIn(qt_Smoke->methodNames[methodRef.name]) != -1 && methodRef.numArgs == 0) { \
+			} else if (predicate_re.indexIn(s->methodNames[methodRef.name]) != -1 && methodRef.numArgs == 0) { \
 				rb_ary_push(result, rb_str_new2((predicate_re.cap(2).toLower() + predicate_re.cap(3) + "?").toLatin1())); \
-			} else if (set_re.indexIn(qt_Smoke->methodNames[methodRef.name]) != -1 && methodRef.numArgs == 1) { \
+			} else if (set_re.indexIn(s->methodNames[methodRef.name]) != -1 && methodRef.numArgs == 1) { \
 				rb_ary_push(result, rb_str_new2((set_re.cap(2).toLower() + set_re.cap(3) + "=").toLatin1())); \
 			} else { \
-				rb_ary_push(result, rb_str_new2(qt_Smoke->methodNames[methodRef.name])); \
+				rb_ary_push(result, rb_str_new2(s->methodNames[methodRef.name])); \
 			} \
 		}
  
@@ -697,13 +701,13 @@ findAllMethodNames(VALUE /*self*/, VALUE result, VALUE classid, VALUE flags_valu
         if (icmp == 0) {
  			for (Smoke::Index i=methmin ; i <= methmax ; i++) {
 				Smoke::Index ix= s->methodMaps[i].method;
-				if (ix >= 0) {	// single match
-					Smoke::Method &methodRef = s->methods[ix];
+				if (ix > 0) {	// single match
+					const Smoke::Method &methodRef = s->methods[ix];
 					PUSH_QTRUBY_METHOD
 				} else {		// multiple match
 					ix = -ix;		// turn into ambiguousMethodList index
 					while (s->ambiguousMethodList[ix]) {
-						Smoke::Method &methodRef = s->methods[s->ambiguousMethodList[ix]];
+						const Smoke::Method &methodRef = s->methods[s->ambiguousMethodList[ix]];
 						PUSH_QTRUBY_METHOD
 						ix++;
 					}
@@ -1020,36 +1024,36 @@ static QRegExp * rx = 0;
 				}			
 			} else if (staticType == "bool") {
 				arg->argType = xmoc_bool;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "int") {
 				arg->argType = xmoc_int;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "uint") {
 				arg->argType = xmoc_uint;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "long") {
 				arg->argType = xmoc_long;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "ulong") {
 				arg->argType = xmoc_ulong;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "double") {
 				arg->argType = xmoc_double;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "char*") {
 				arg->argType = xmoc_charstar;
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			} else if (staticType == "QString") {
 				arg->argType = xmoc_QString;
 				name += "*";
-				smoke = qt_Smoke;
+				smoke = qtcore_Smoke;
 				typeId = smoke->idType(name.constData());
 			}
 
@@ -1228,14 +1232,14 @@ value_to_type_flag(VALUE ruby_value)
 VALUE prettyPrintMethod(Smoke::Index id) 
 {
     VALUE r = rb_str_new2("");
-    Smoke::Method &meth = qt_Smoke->methods[id];
-    const char *tname = qt_Smoke->types[meth.ret].name;
+    const Smoke::Method &meth = qtcore_Smoke->methods[id];
+    const char *tname = qtcore_Smoke->types[meth.ret].name;
     if(meth.flags & Smoke::mf_static) rb_str_catf(r, "static ");
     rb_str_catf(r, "%s ", (tname ? tname:"void"));
-    rb_str_catf(r, "%s::%s(", qt_Smoke->classes[meth.classId].className, qt_Smoke->methodNames[meth.name]);
+    rb_str_catf(r, "%s::%s(", qtcore_Smoke->classes[meth.classId].className, qtcore_Smoke->methodNames[meth.name]);
     for(int i = 0; i < meth.numArgs; i++) {
 	if(i) rb_str_catf(r, ", ");
-	tname = qt_Smoke->types[qt_Smoke->argumentList[meth.args+i]].name;
+	tname = qtcore_Smoke->types[qtcore_Smoke->argumentList[meth.args+i]].name;
 	rb_str_catf(r, "%s", (tname ? tname:"void"));
     }
     rb_str_catf(r, ")");
