@@ -26,7 +26,7 @@
 #ifndef QT_VERSION_STR
 #define QT_VERSION_STR "Unknown"
 #endif
-#define QTRUBY_VERSION "2.0.3"
+#define QTRUBY_VERSION "2.0.5"
 
 #if !defined RSTRING_LEN
 #define RSTRING_LEN(a) RSTRING(a)->len
@@ -40,13 +40,9 @@
 #if !defined RARRAY_PTR
 #define RARRAY_PTR(a) RARRAY(a)->ptr
 #endif
-
-inline bool operator==(const Smoke::ModuleIndex& a, const Smoke::ModuleIndex& b) {
-	if (a.index == b.index && a.smoke == b.smoke)
-		return true;
-	else
-		return false;
-}
+#if !defined StringValueCStr
+#define StringValueCStr(s) STR2CSTR(s)
+#endif
 
 inline uint qHash(const Smoke::ModuleIndex& mi) {
 	return qHash(mi.index) ^ qHash(mi.smoke);
